module Brain
       (Brain, empty, lookup, member, insert, findLower, findUpper) where
import Prelude (Show, Bool(..), Maybe, ($), (.), return)
import Data.Maybe (fromMaybe, isJust)
import qualified Map as M
import Path (Path)
import Proof (Proof, conclusion)
import Prop (Prop(..))
 
type HalfBrain = M.Map Path (M.Map Path Proof)
 
data Brain = B{byLower :: !HalfBrain, byUpper :: !HalfBrain}
           deriving Show
 
empty :: Brain
empty = B M.empty M.empty
 
lookup :: Prop -> Brain -> Maybe Proof
lookup (SubsetOf p1 p2) brain
  = do m <- M.lookup p1 $ byLower brain
       M.lookup p2 m
 
member :: Prop -> Brain -> Bool
member prop = isJust . lookup prop
 
insert :: Proof -> Brain -> Brain
insert proof brain
  = let SubsetOf p1 p2 = conclusion proof in
      B (M.insert p1
           (M.insert p2 proof $ fromMaybe M.empty $ M.lookup p1 $
              byLower brain)
           $ byLower brain)
        (M.insert p2
           (M.insert p1 proof $ fromMaybe M.empty $ M.lookup p2 $
              byUpper brain)
           $ byUpper brain)
 
findLower :: Path -> Brain -> [(Path, Proof)]
findLower p brain
  = fromMaybe [] $
      do m <- M.lookup p $ byLower brain
         return $ M.toList m
 
findUpper :: Path -> Brain -> [(Path, Proof)]
findUpper p brain
  = fromMaybe [] $
      do m <- M.lookup p $ byUpper brain
         return $ M.toList m
