module Map (Map, empty, lookup, member, insert, toList, elems)
       where
import Prelude
       (Ord, Show, Bool(..), Maybe, Ordering(..), (.), compare, map,
        return, snd)
import Control.Monad (mzero)
import Data.Maybe (isJust)
 
data Color = R
           | B
           deriving Show
 
data Map k v = E
             | T !Color !(Map k v) !k !v !(Map k v)
             deriving Show
 
empty :: Map k v
empty = E
 
lookup :: (Ord k) => k -> Map k v -> Maybe v
lookup k
  = let lookup' (E) = mzero
        lookup' (T _color a xk xv b)
          = case compare k xk of
                LT -> lookup' a
                EQ -> return xv
                GT -> lookup' b
      in lookup'
 
member :: (Ord k) => k -> Map k v -> Bool
member k = isJust . lookup k
 
balanceLT ::
          (Ord k) => Color -> Map k v -> k -> v -> Map k v -> Map k v
balanceLT (B) (T R (T R a xk xv b) yk yv c) zk zv d
  = T R (T B a xk xv b) yk yv (T B c zk zv d)
balanceLT (B) (T R a xk xv (T R b yk yv c)) zk zv d
  = T R (T B a xk xv b) yk yv (T B c zk zv d)
balanceLT color a xk xv b = T color a xk xv b
 
balanceGT ::
          (Ord k) => Color -> Map k v -> k -> v -> Map k v -> Map k v
balanceGT (B) a xk xv (T R (T R b yk yv c) zk zv d)
  = T R (T B a xk xv b) yk yv (T B c zk zv d)
balanceGT (B) a xk xv (T R b yk yv (T R c zk zv d))
  = T R (T B a xk xv b) yk yv (T B c zk zv d)
balanceGT color a xk xv b = T color a xk xv b
 
blacken :: Map k v -> Map k v
blacken (E) = E
blacken (T _color a xk xv b) = T B a xk xv b
 
insert :: (Ord k) => k -> v -> Map k v -> Map k v
insert xk xv
  = let insert' (E) = T R E xk xv E
        insert' (T color a yk yv b)
          = case compare xk yk of
                LT -> balanceLT color (insert' a) yk yv b
                EQ -> T color a xk xv b
                GT -> balanceGT color a yk yv (insert' b)
      in blacken . insert'
 
toList' :: [(k, v)] -> Map k v -> [(k, v)]
toList' kvs (E) = kvs
toList' kvs (T _color a xk xv b)
  = toList' ((xk, xv) : toList' kvs b) a
 
toList :: Map k v -> [(k, v)]
toList = toList' []
 
elems :: Map k v -> [v]
elems = map snd . toList
