module Path
       (Path, id_, format, dom, codom, type_, pathE, outgoing, incoming,
        inv, comp, zero, kernelInv, kernel, invZero, image, dual,
        complexity, unKernelInv, unImage)
       where
import Prelude
       (Eq, Ord, Show, Int, Maybe, ShowS, (+), (++), (.), (==), id,
        return)
import Control.Monad (mzero)
import Graph
       (Vertex(..), dualV, Edge(..), formatE, domE, codomE, outgoingE,
        incomingE, initialE, terminalE, dualE)
 
default (Int)
 
data Direction = D
               | Dinv
               deriving (Eq, Ord, Show)
 
formatD :: Direction -> ShowS
formatD d
  = case d of
        D -> id
        Dinv -> ("\8315\185" ++)
 
invD :: Direction -> Direction
invD d
  = case d of
        D -> Dinv
        Dinv -> D
 
data Path = Pid !Vertex
          | Pcomp !Path !Edge !Direction
          deriving (Eq, Ord, Show)
 
id_ :: Vertex -> Path
id_ = Pid
 
format' :: Path -> ShowS
format' p
  = case p of
        Pid _v -> id
        Pcomp p' e d -> format' p' . formatE e . formatD d . (' ' :)
 
format :: Path -> ShowS
format p
  = case p of
        Pid _v -> ("1" ++)
        Pcomp p' e d -> format' p' . formatE e . formatD d
 
dom :: Path -> Vertex
dom p
  = case p of
        Pid v -> v
        Pcomp _p' e d
          -> (case d of
                  D -> domE
                  Dinv -> codomE)
               e
 
codom :: Path -> Vertex
codom p
  = case p of
        Pid v -> v
        Pcomp p' _e _d -> codom p'
 
type_ :: Path -> (Vertex, Vertex)
type_ p = (dom p, codom p)
 
comp_ :: Path -> Edge -> Direction -> Path
comp_ p' e d
  | dom p' ==
      (case d of
           D -> codomE
           Dinv -> domE)
        e
    = Pcomp p' e d
 
pathE :: Edge -> Path
pathE e = comp_ (id_ (codomE e)) e D
 
invPathE :: Edge -> Path
invPathE e = comp_ (id_ (domE e)) e Dinv
 
outgoing :: Vertex -> [Path]
outgoing v
  = [pathE e | e <- outgoingE v] ++ [invPathE e | e <- incomingE v]
 
incoming :: Vertex -> [Path]
incoming v
  = [pathE e | e <- incomingE v] ++ [invPathE e | e <- outgoingE v]
 
compInv :: Path -> Path -> Path
compInv p' q
  | dom p' == dom q =
    case q of
        Pid _v -> p'
        Pcomp q' e d -> compInv (comp_ p' e (invD d)) q'
 
inv :: Path -> Path
inv q = compInv (id_ (dom q)) q
 
comp :: Path -> Path -> Path
comp p2 p1
  = case p1 of
        Pid v | dom p2 == v -> p2
        Pcomp p1' e d -> comp_ (comp p2 p1') e d
 
zero :: Vertex -> Path
zero = pathE . initialE
 
kernelInv :: Path -> Path
kernelInv p = comp p (zero (dom p))
 
kernel :: Path -> Path
kernel = kernelInv . inv
 
invZero :: Vertex -> Path
invZero = invPathE . terminalE
 
image :: Path -> Path
image p = comp p (invZero (dom p))
 
compDual :: Path -> Path -> Path
compDual p' q
  | dom p' == dualV (dom q) =
    case q of
        Pid _v -> p'
        Pcomp q' e d -> compDual (comp_ p' (dualE e) d) q'
 
dual :: Path -> Path
dual q = compDual (id_ (dualV (dom q))) q
 
complexity :: Path -> Int
complexity p
  = case p of
        Pid _v -> 0
        Pcomp p' e _d
          -> (case e of
                  E0 _v _cov -> 0
                  _e -> 1)
               + complexity p'
 
unKernelInv :: Path -> Maybe Path
unKernelInv p
  = case p of
        Pcomp p' (E0 V0 _cov) D -> return p'
        _ -> mzero
 
unImage :: Path -> Maybe Path
unImage p
  = case p of
        Pcomp p' (E0 _v V0) Dinv -> return p'
        _ -> mzero
