module Pic
       (Pic, empty, glyph, up, right, over, formatPic,
        picAndWidthFromString, horizontalRule)
       where
import Prelude
       (Show, Char, Int, ShowS, String, (&&), (+), (-), (.), (<), (<=),
        (==), fst, id, otherwise)
import Data.Char (isPrint)
import Data.List (sortBy)
import Data.Ord (comparing)
 
default (Int)
 
data Pic = Empty
         | Glyph Char
         | Up Int Pic
         | Right Int Pic
         | Over Pic Pic
         deriving Show
 
empty :: Pic
empty = Empty
 
glyph :: Char -> Pic
glyph c | isPrint c = Glyph c
 
up :: Int -> Pic -> Pic
up dy | 0 <= dy = Up dy
 
right :: Int -> Pic -> Pic
right dx | 0 <= dx = Right dx
 
over :: Pic -> Pic -> Pic
over = Over
 
type Command = ((Int, Int), Char)
 
commands' :: [Command] -> Int -> Int -> Pic -> [Command]
commands' cmds' y x pic
  = case pic of
        Empty -> cmds'
        Glyph c -> ((y, x), c) : cmds'
        Up dy pic' -> commands' cmds' (dy + y) x pic'
        Right dx pic' -> commands' cmds' y (dx + x) pic'
        Over pic2 pic1
          -> let cmds = commands' cmds' y x pic1 in commands' cmds y x pic2
 
commands :: Pic -> [Command]
commands = commands' [] 0 0
 
formatCommands :: ShowS -> Int -> Int -> [Command] -> ShowS
formatCommands s y x cmds
  = case cmds of
        [] -> s . ('\n' :)
        ((y', x'), c) : cmds'
          | y < y' -> formatCommands id (1 + y) 0 cmds . s . ('\n' :)
          | y == y' && x < x' -> formatCommands (s . (' ' :)) y (1 + x) cmds
          | y == y' && x == x' -> formatCommands (s . (c :)) y (1 + x) cmds'
          | otherwise -> formatCommands s y x cmds'
 
formatPic :: Pic -> ShowS
formatPic
  = formatCommands id 0 0 . sortBy (comparing fst) . commands
 
picAndWidthFromString :: String -> (Pic, Int)
picAndWidthFromString cs
  = case cs of
        [] -> (empty, 0)
        c : cs'
          -> let (pic', dx') = picAndWidthFromString cs' in
               (glyph c `over` right 1 pic', 1 + dx')
 
horizontalRule :: Int -> Pic
horizontalRule n
  | 0 <= n =
    if 0 == n then empty else
      glyph '\9472' `over` right 1 (horizontalRule (n - 1))
