module Prop
       (Prop(..), subsetOf, formatProp, domProp, codomProp, typeProp,
        equalTo, exact, invProp, dualProp, complexityProp)
       where
import Prelude (Eq, Ord, Show, Int, ShowS, (++), (.), (==), max)
import Graph (Vertex)
import Path
       (Path, format, dom, codom, type_, inv, zero, kernelInv, kernel,
        invZero, image, dual, complexity)
 
data Prop = SubsetOf !Path !Path
          deriving (Eq, Ord, Show)
 
subsetOf :: Path -> Path -> [Prop]
subsetOf p1 p2 | type_ p1 == type_ p2 = [SubsetOf p1 p2]
 
formatProp :: Prop -> ShowS
formatProp (SubsetOf p1 p2)
  = format p1 . (" \8838 " ++) . format p2
 
domProp :: Prop -> Vertex
domProp (SubsetOf p1 _p2) = dom p1
 
codomProp :: Prop -> Vertex
codomProp (SubsetOf p1 _p2) = codom p1
 
typeProp :: Prop -> (Vertex, Vertex)
typeProp (SubsetOf p1 _p2) = type_ p1
 
equalTo :: Path -> Path -> [Prop]
equalTo p1 p2 = subsetOf p1 p2 ++ subsetOf p2 p1
 
wellDefined :: Path -> [Prop]
wellDefined p = subsetOf (kernelInv p) (zero (codom p))
 
oneToOne :: Path -> [Prop]
oneToOne = wellDefined . inv
 
onto :: Path -> [Prop]
onto p = subsetOf (invZero (codom p)) (image p)
 
total :: Path -> [Prop]
total = onto . inv
 
exact' :: [Path] -> [Prop]
exact' ps
  = case ps of
        [] -> []
        p1 : ps'
          -> total p1 ++ wellDefined p1 ++
               case ps' of
                   [] -> onto p1
                   p2 : _ps'' -> equalTo (image p1) (kernel p2) ++ exact' ps'
 
exact :: [Path] -> [Prop]
exact ps
  = case ps of
        [] -> []
        p : _ps' -> oneToOne p ++ exact' ps
 
invProp :: Prop -> Prop
invProp (SubsetOf p1 p2) = SubsetOf (inv p1) (inv p2)
 
dualProp :: Prop -> Prop
dualProp (SubsetOf p1 p2) = SubsetOf (dual p2) (dual p1)
 
complexityProp :: Prop -> Int
complexityProp (SubsetOf p1 p2)
  = max (complexity p1) (complexity p2)
