module Snake (axioms, hypotheses, goals) where
import Prelude ((++))
import Graph (vertices, Edge(..))
import Path
       (Path, pathE, incoming, inv, comp, zero, kernelInv, invZero, image)
import Prop (Prop, subsetOf, equalTo, exact)
 
alpha :: Path
alpha = pathE Ealpha
 
beta :: Path
beta = pathE Ebeta
 
gamma :: Path
gamma = pathE Egamma
 
delta :: Path
delta = pathE Edelta
 
epsilon :: Path
epsilon = pathE Eepsilon
 
zeta :: Path
zeta = pathE Ezeta
 
eta :: Path
eta = pathE Eeta
 
theta :: Path
theta = pathE Etheta
 
lambda :: Path
lambda = pathE Elambda
 
mu :: Path
mu = pathE Emu
 
nu :: Path
nu = pathE Enu
 
xi :: Path
xi = pathE Exi
 
pi :: Path
pi = pathE Epi
 
rho :: Path
rho = pathE Erho
 
sigma :: Path
sigma = pathE Esigma
 
tau :: Path
tau = pathE Etau
 
phi :: Path
phi = pathE Ephi
 
omega :: Path
omega = pathE Eomega
 
axioms :: [Prop]
axioms
  = [prop | v <- vertices, prop <- subsetOf (zero v) (invZero v)] ++
      [prop | v <- vertices, p' <- incoming v,
       p <- [kernelInv p', image p'],
       prop <- subsetOf (zero v) p ++ subsetOf p (invZero v)]
 
hypotheses :: [Prop]
hypotheses
  = equalTo (comp lambda zeta) (comp nu theta) ++
      equalTo (comp mu eta) (comp xi lambda)
      ++ exact [gamma, theta, pi]
      ++ exact [delta, lambda, rho]
      ++ exact [epsilon, mu, sigma]
      ++ exact [zeta, eta]
      ++ exact [nu, xi]
      ++ equalTo alpha (comp (comp (inv delta) zeta) gamma)
      ++ equalTo beta (comp (comp (inv epsilon) eta) delta)
      ++ equalTo tau (comp (comp rho nu) (inv pi))
      ++ equalTo phi (comp (comp sigma xi) (inv rho))
      ++
      equalTo omega
        (comp (comp (comp (comp pi (inv nu)) lambda) (inv eta)) epsilon)
 
goals :: [Prop]
goals
  = equalTo (comp delta alpha) (comp zeta gamma) ++
      equalTo (comp epsilon beta) (comp eta delta)
      ++ equalTo (comp rho nu) (comp tau pi)
      ++ equalTo (comp sigma xi) (comp phi rho)
      ++ exact [alpha, beta, omega, tau, phi]
